// $Id: Parameter.cc 73 2006-02-14 22:42:33Z jari $

#include "Parameter.h"

#include <cstdlib>
#include <iostream>
#include <string>

#include <c++_tools/utility/FileIO.h>


namespace theplu {
namespace wenni {
namespace nnifileconverter {

	Parameter::Parameter(const int argc,const char *argv[])
	{
		using namespace std;

		defaults();

		for (int i=1; i<argc; i++) {
			bool ok=false;
			string myargv(argv[i]);
			if (myargv==string("-help") || myargv==string("-h")) {
				help();
				exit(0);			// always exit after printing help
			}
			if (myargv==string("-beta"))
				if ((i+1)<argc) {
					beta_=atof(argv[++i]);
					ok=true;
				}
			if (myargv==string("-bg1"))
				if ((i+1)<argc) {
					bg1_=argv[++i];
					ok=true;
				}
			if (myargv==string("-bg2"))
				if ((i+1)<argc) {
					bg2_=argv[++i];
					ok=true;
				}
			if (myargv==string("-bgstd1"))
				if ((i+1)<argc) {
					bgstd1_=argv[++i];
					ok=true;
				}
			if (myargv==string("-bgstd2"))
				if ((i+1)<argc) {
					bgstd2_=argv[++i];
					ok=true;
				}
			if (myargv==string("-datatype"))
				if ((i+1)<argc) {
					datatype_=argv[++i];
					ok=true;
				}
			if (myargv==string("-fg1"))
				if ((i+1)<argc) {
					fg1_=argv[++i];
					ok=true;
				}
			if (myargv==string("-fg2"))
				if ((i+1)<argc) {
					fg2_=argv[++i];
					ok=true;
				}
			if (myargv==string("-logratio"))
				if ((i+1)<argc) {
					logratio_=argv[++i];
					ok=true;
				}
			if (myargv==string("-weight"))
				if ((i+1)<argc) {
					weight_=argv[++i];
					ok=true;
				}
			if (!ok)
				std::cerr << "# Parameter::Parameter     Invalid option: "
									<< argv[i] << '\n';
		}

		analyse();
	}

	void Parameter::analyse(void)
	{
		using namespace theplu::utility;
		bool allok=true;

		if (beta_<0) {
			std::cerr << "Negativ beta is not allowed " << beta_ << std::endl;
			allok=false;
		}

		if (datatype_==std::string("raw")) {
			if (bg1_.empty()) {
				std::cerr << "No file for bg1 given" << std::endl;
				allok=false;
			}
			else if (FileIO().access_rights(bg1_,"r")) {
				std::cerr << "Cannot access channel 1 background input file " << bg1_
									<< std::endl;
				allok=false;
			}

			if (bg2_.empty()) {
				std::cerr << "No file for bg2 given" << std::endl;
				allok=false;
			}
			else if (FileIO().access_rights(bg2_,"r")) {
				std::cerr << "Cannot access channel 2 background input file " << bg2_
									<< std::endl;
				allok=false;
			}
		}

		if (bgstd1_.empty()) {
			std::cerr << "No file for bgstd1 given" << std::endl;
			allok=false;
		}
		else if (FileIO().access_rights(bgstd1_,"r")) {
			std::cerr << "Cannot access channel 1 background std input file "
								<< bgstd1_ << std::endl;
			allok=false;
		}

		if (bgstd2_.empty()) {
			std::cerr << "No file for bgstd2 given" << std::endl;
			allok=false;
		}
		else if (FileIO().access_rights(bgstd2_,"r")) {
			std::cerr << "Cannot access channel 2 background std input file "
								<< bgstd2_ << std::endl;
			allok=false;
		}

		if (datatype_!=std::string("derived") && datatype_!=std::string("raw")) {
			std::cerr << "Illegal datatype value: " << datatype_ << std::endl;
			allok=false;
		}

		if (fg1_.empty()) {
			std::cerr << "No file for fg1 given" << std::endl;
			allok=false;
		}
		else if (FileIO().access_rights(fg1_,"r")) {
			std::cerr << "Cannot access channel 1 foreground input file " << fg1_
								<< std::endl;
			allok=false;
		}

		if (fg2_.empty()) {
			std::cerr << "No file for fg2 given" << std::endl;
			allok=false;
		}
		else if (FileIO().access_rights(fg2_,"r")) {
			std::cerr << "Cannot access channel 2 foreground input file " << fg2_
								<< std::endl;
			allok=false;
		}

		if (logratio_.empty()) {
			std::cerr << "No file for logratio given" << std::endl;
			allok=false;
		}
		/*
		else if (FileIO().access_rights(logratio_,"w")) {
			std::cerr << "No write rights for output file " << logratio_ << std::endl;
			allok=false;
		}
		*/

		if (weight_.empty()) {
			std::cerr << "No file for weight given" << std::endl;
			allok=false;
		}
		/*
		else if (FileIO().access_rights(weight_,"w")) {
			std::cerr << "No write rights for output file " << weight_ << std::endl;
			allok=false;
		}
		*/

		if (!allok)
			std::exit(-1);
	}



	void Parameter::defaults(void)
	{
		beta_=0.6;
		datatype_=std::string("raw");
		logratio_=std::string("logratio.data");
		weight_=std::string("weight.data");
	}



	void Parameter::help(void) const
	{	
		std::cout << "Convert files created by BaseFileConverter to logratios\n"
							<< "and weights\n"
							<< "Usage: filter -option option\n"
							<< "\nInputs:\n"
							<< "\t-beta     beta value of the snr to weight calculation\n"
							<< "\t          [default: " << beta_ << "]\n"
							<< "\t-datatype Values 'raw' or 'derived' . When 'derived' is\n"
							<< "\t          given, only foreground files are expected, and\n"
							<< "\t          background files ignored (cf. options -fg1,\n"
							<< "\t          -fg2, -bg1, and -bg2.\n"
							<< "\t          [" << datatype_ << "]\n"
							<< "\t-fg1      input file first foreground\n"
							<< "\t-bg1      input file first background\n"
							<< "\t-bgstd1   input file first background standard deviation\n"
							<< "\t-fg2      input file second foreground\n"
							<< "\t-bg2      input file second background\n"
							<< "\t-bgstd2   input file second background standard deviation\n"
							<< "\t-logratio output file logratio (first/second)\n"
							<< "\t          [" << logratio_ << "\n"
							<< "\t-weight   output file weight\n"
							<< "\t          [" << weight_ << "]\n";
	}

}}} // of namespace nnifileconverter, wenni and namespace theplu
