// $Id: WeNNI.h 69 2006-02-11 00:44:22Z jari $

#ifndef _theplu_utility_wenni_
#define _theplu_utility_wenni_ 

#include <c++_tools/utility/NNI.h>

#include <c++_tools/gslapi/matrix.h>

#include <iostream>



namespace theplu {
namespace utility {

	///
	/// WeNNI is a continuous weights generalization of the (binary
	/// weights) kNNI algorithm presented by Troyanskaya et al. A
	/// reference to this paper is found in the NNI document referred to
	/// in the NNI class documentation. The NNI document also describes
	/// WeNNI in depth.
	///
	/// @see NNI and kNNI
	///
	class WeNNI : public NNI
	{
  public:
		///
		/// Constructor
		///
		WeNNI(const gslapi::matrix& matrix,const gslapi::matrix& weight,
					const u_int neighbours);

		///
		/// Perform WeNNI on data in \a matrix with continuous uncertainty
		/// weights in \a weight using \a neighbours for the new impute
		/// value.
		///
		u_int estimate(void);

		///
		/// @return A const reference to imputed_data_raw.
		///
		const gslapi::matrix& imputed_data_raw(void) const 
		{ return imputed_data_raw_; }



	private:

		gslapi::matrix imputed_data_raw_;
  };

}} // of namespace utility and namespace theplu

#endif
