// $Id: Data.cc 69 2006-02-11 00:44:22Z jari $

#include "Data.h"

#include <fstream>
#include <iostream>

#include <basefile.h>

namespace theplu {
namespace wenni {


	Data::Data(BASE::BaseFileSection& bfs,const std::string& colname, 
						 const std::string& output_file)
		: column_name_(colname), output_file_(output_file)
	{
		BASE::FieldList columns, assayFields, assays;
		std::string assay_names;
		if( !bfs.findFieldList("columns", columns) ||
				!bfs.findFieldList("assayFields", assayFields) ||
				!bfs.findFieldList("assays", assays) ||
				!bfs.findStringOpt("assays", assay_names)) {
			std::cerr << "Missing header columns/assayFields/assays\n";
			exit(-1);
		}
		nof_assays_=assays.size();
		column_offset_=columns.size()-1;
		nof_fields_per_assay_=assayFields.size();
		if (!bfs.findFieldPos(column_name_,column_number_,assayFields)) {
			std::cerr << "Cannot find assayField " << column_name_ << std::endl;
			exit(-1);
		}
		total_nof_columns_= column_offset_ + nof_fields_per_assay_*nof_assays_;
	}



	void Data::push_back(const std::vector<std::string>& vals)
	{
		data_.push_back(std::vector<std::string>(nof_assays_));
		for(int i=0; i<nof_assays_; i++){
			if(vals[column_offset_+nof_fields_per_assay_*i+column_number_].empty()){
				// In a Base File missing values are given by a blank. To be
				// able to load the data into a gsl matrix missing values are
				// changed to NaNs.
				(*data_.rbegin())[i]="NaN";
			}
			else
				(*data_.rbegin())[i]=vals[column_offset_ + nof_fields_per_assay_ * i +
																	column_number_];
		}
	}



	void Data::write_matrix(void)
	{
		std::filebuf fb;
		fb.open (output_file_.c_str(),std::ios::out);
		std::ostream s(&fb);
		for (size_t i=0; i<data_.size(); i++) {
			for (size_t j=0; j<data_[i].size()-1; j++)
				s << data_[i][j] << "\t";
			s << data_[i][data_[i].size()-1] << std::endl;
		}
		fb.close();
	}


}} // of namespace wenni and namespace theplu
