// $Id: Parameter.cc 69 2006-02-11 00:44:22Z jari $

#include "Parameter.h"

#include <iostream>
#include <string>

#include <c++_tools/utility/FileIO.h>


namespace theplu {
namespace wenni {

	Parameter::Parameter(const int argc,const char *argv[])
	{
		using namespace std;
		// check if help requested
		for (int i=1; i<argc; i++)
			if (string(argv[i])==string("-help") || string(argv[i])==string("-h")) {
				help();
				exit(0);			// always exit after printing help
			}

		if (argc<2) {
			cerr << "Too few parameters." << '\n';
			help();
			exit(-1);			// always exit after printing help
		}

		input_file_=argv[1];
		show_=false;
		if (string(argv[2])==string("-show"))
			show_=true;
		else if (argc>2) {
			std::string type("-assayFields");
			for (int i=3; i<argc; i++) {
				if (string(argv[i])==string("-assayFields") ||
						string(argv[i])==string("-columns"))
					type = argv[i];
				else {
					output_file_.push_back(string(argv[2]) + string(argv[i]) +
																 string(".data"));
					property_.push_back(argv[i]);
					type_.push_back(type);
				}
			}
		}
		else {
			cerr << "Too few parameters." << '\n';
			help();
			exit(0);			// always exit after printing help
		}

		analyse();
	}



	void Parameter::analyse(void)
	{
		if (theplu::utility::FileIO().access_rights(input_file(),"r")) {
			std::cerr << "Cannot access input file " << input_file() << std::endl;
			std::exit(-1);
		}
	}



	void Parameter::help(void) const
	{
		std::cout << "Usage: BaseFileConverter <input> <output> <type> "
							<< "<name>... <type2> <name>\n\t<input> = data file exported "
							<< "from base (http://base.thep.lu.se/)\n\t<output> = "
							<< "suffix for names of generated files. File name is "
							<< "<output>+<name>+.data\n\t<type> = type of following names, "
							<< "e.g. -assayFields\n\t<name> = name of field to convert, "
							<< "e.g. l2ratio1_2\n\t\nfor available options:\n" 
							<< "BaseFileConverter <input> -show\n\nfor this help\n"
							<< "BaseFileConverter -help\n";
	}

}} // of namespace wenni and namespace theplu
