// $Id: Parameter.cc 88 2006-04-04 11:06:13Z jari $

#include "Parameter.h"
#include "nni.h"
#include <config.h>	// this header file is created by configure

#include <iostream>
#include <string>

#include <c++_tools/utility/FileIO.h>


namespace theplu {
namespace wenni {

	Parameter::Parameter(const int argc,const char *argv[])
	{
		using namespace std;
		// check if help requested
		for (int i=1; i<argc; i++)
			if (string(argv[i])==string("-help") || string(argv[i])==string("-h")) {
#include "nni.hlp"
				exit(0);			// always exit after printing help
			}

		defaults();

		for (int i=1; i<argc; i++) {
			bool ok=false;
			string myargv(argv[i]);
			if (myargv==string("-beta"))
				if (++i<argc) {
					beta_=atof(argv[i]);
					ok=true;
				}
			if (myargv==string("-data"))
				if (++i<argc) {
					data_file_=argv[i];
					ok=true;
				}
			if (myargv==string("-neighbours"))
				if (++i<argc) {
					neighbours_=atoi(argv[i]);
					ok=true;
				}
			if (myargv==string("-nni_algorithm"))
				if (++i<argc) {
					ok=true;
					if (argv[i]==string("kNNI"))
						nni_algorithm_=kNNI;
					else
						if (argv[i]==string("WeNNI"))
							nni_algorithm_=WeNNI;
						else
							ok=false;
				}
			if (myargv=="--version") {
					version();
					exit(0);
			}
			if (myargv==string("-weight"))
				if (++i<argc) {
					weight_file_=argv[i];
					ok=true;
				}
			if (myargv==string("-weight_cutoff"))
				if (++i<argc) {
					weight_cutoff_=atof(argv[i]);
					ok=true;
				}
			if (myargv==string("-weight_is_snr")) {
					weight_is_snr_=!weight_is_snr_;
					ok=true;
				}

			if (!ok)
				std::cerr << "# Parameter::Parameter     Invalid option:"
									<< argv[i] << std::endl;
		}

		analyse();
	}



	void Parameter::analyse(void)
	{
		using namespace theplu::utility;
		bool ok=true;
		if (FileIO().access_rights(data_file(),"r")) {
			std::cerr << "Cannot access data file " << data_file() << std::endl;
			ok=false;
		}
		if (FileIO().access_rights(weight_file(),"r")) {
			std::cerr << "Cannot access flag file " << weight_file() << std::endl;
			ok=false;
		}
		if (beta_<0) {
			std::cerr << "Negative beta (" << beta_ << ") is out of bounds."
								<< std::endl;
			ok=false;
		}
		if (weight_cutoff_ && (nni_algorithm()==WeNNI))
			std::cerr << "Algo WeNNI set, non-zero weight cutoff (" << weight_cutoff_
								<< ") ignorded." << std::endl;
		if ((nni_algorithm()<kNNI) ||
				(nni_algorithm()>WeNNI)) {
			std::cerr << "Algo out of bounds " << nni_algorithm() << std::endl;
			ok=false;
		}
		if (!ok)
			std::exit(-1);
	}



	void Parameter::defaults(void)
	{
		beta_=0.6;
		data_file_="data.data";
		neighbours_=3;
		nni_algorithm_=WeNNI;
		weight_cutoff_=0;
		weight_file_="weight.data";
		weight_is_snr_=false;
	}



	void Parameter::version(void) const
	{
		using namespace std;
		cout << PACKAGE_STRING
				 << "\nCopyright (C) 2005-2006 Jari Häkkinen and Peter Johansson.\n\n"
				 << "This is free software; see the source for copying conditions.\n"
				 << "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR\n"
				 << "A PARTICULAR PURPOSE." << endl;
	}

}} // of namespace wenni and namespace theplu
