// $Id: nni.cc 69 2006-02-11 00:44:22Z jari $

#include "nni.h"

#include <iostream>
#include <fstream>

#include <c++_tools/utility/kNNI.h>
#include <c++_tools/utility/WeNNI.h>

#include "Parameter.h"
#include "weight.h"

void make_weights_binary(theplu::gslapi::matrix& weight,double cutoff);

int main(const int argc,const char* argv[])
{
	using namespace theplu;
	wenni::Parameter option(argc,argv);

	std::ifstream data_stream(option.data_file().c_str());
	std::ifstream weight_stream(option.weight_file().c_str());
	gslapi::matrix data(data_stream);
	gslapi::matrix weight(weight_stream);
	data_stream.close();
	weight_stream.close();

	if (option.weight_is_snr())
		weight=wenni::weight::weight_SNR(weight,option.beta());

	utility::NNI* nni=NULL;
	switch (option.nni_algorithm()) {
	case wenni::kNNI:
		make_weights_binary(weight,option.weight_cutoff());
		nni=new utility::kNNI(data,weight,option.neighbours());
		break;
	case wenni::WeNNI:
		nni=new utility::WeNNI(data,weight,option.neighbours());
		break;
	}
	nni->estimate();
	std::cout << nni->imputed_data() << '\n';

	delete nni;

	exit(0);				// normal exit
}

void make_weights_binary(theplu::gslapi::matrix& weight,double cutoff)
{
	for (size_t i=0; i<weight.rows(); ++i)
		for (size_t j=0; j<weight.columns(); ++j)
			weight(i,j)=(weight(i,j)>cutoff ? 1 : 0);
}
