// $Id: FileIO.h 69 2006-02-11 00:44:22Z jari $

#ifndef _theplu_utility_fileio_
#define _theplu_utility_fileio_ 

#include <string>

#include <unistd.h>

namespace theplu {
namespace utility {

	///
	/// FileIO is useful for many common task on files.
	///
	class FileIO {
	public:

		///
		/// Check if access permissions match \a mode. \a mode must be
		/// given as r, w, x, or combinations of these letters.
		///
		/// @return On success (all requested permissions granted), zero
		/// is returned. On error (at least one bit in mode asked for a
		/// permission that is denied, or some other error occurred), -1
		/// is returned, and errno is set appropriately.
		///
		/// @see access(2)
		///
		/// @note Checking for write permissions will fail if the file
		/// does not exists. This should be fixed so that when a file does
		/// not exist, permissions to create the file should be returned.
		///
		int access_rights(const std::string& path,const std::string& bits) const;

		///
		/// Check whether \a file exists.
		///
		/// @return True if \a file exists, false otherwise.
		///
		/// @see access(2)
		///
		inline bool file_exists(const std::string& file) const
			{ return !access(file.c_str(),F_OK); }
	};

}} // of namespace utility and namespace theplu

#endif
