// $Id: kNNI.cc 69 2006-02-11 00:44:22Z jari $

#include <c++_tools/utility/kNNI.h>

#include <algorithm>
#include <cmath>
#include <fstream>
#include <vector>

#include <c++_tools/utility/stl_utility.h>

namespace theplu {
namespace utility {

	kNNI::kNNI(const gslapi::matrix& matrix,const gslapi::matrix& flag,
						 const u_int neighbours)
		: NNI(matrix,flag,neighbours)
	{
		for (unsigned int i=0; i<weight_.rows(); i++)
			for (unsigned int j=0; j<weight_.columns(); j++)
				if (!weight_(i,j)) {
					mv_rows_.push_back(i);
					break;
				}
		//estimate();
	}



	// \hat{x_{ij}}=\frac{ \sum_{k=1,N} \frac{x_{kj}}{d_{ki}} }
	//                   { \sum_{k=1,N} \frac{1     }{d_{ki}} },
	// where N is defined in the paper cited in the NNI class definition
	// documentation.
	u_int kNNI::estimate(void)
	{
		using namespace std;
		for (unsigned int i=0; i<mv_rows_.size(); i++) {
			// Jari, avoid copying in next line
			vector<pair<u_int,double> > distance=calculate_distances(mv_rows_[i]);
 			sort(distance.begin(),distance.end(),
								pair_value_compare<u_int,double>());
			for (unsigned int j=0; j<data_.columns(); j++)
				if (!weight_(mv_rows_[i],j)) {
					vector<u_int> knn=nearest_neighbours(j,distance);
					double new_value=0.0;
					double norm=0.0;
					for (vector<u_int>::const_iterator k=knn.begin(); k!=knn.end(); k++){
						// Jari, a small number needed here, use something standardized.
						// Avoid division with zero (perfect match vectors)
						double d=(distance[*k].second ? distance[*k].second : 1e-10);
						new_value+=data_(distance[*k].first,j)/d;
						norm+=1.0/d;
					}
					// No impute if no contributions from neighbours.
					if (norm)
						imputed_data_(mv_rows_[i],j)=new_value/norm;
					else {
						not_imputed_.push_back(i);
						// if norm is zero for one column it is zero for all columns
						// having zero weight
						break;
					}
				}
		}
		return not_imputed_.size();
	}


}} // of namespace utility and namespace theplu
