// $Id: stl_utility.h 69 2006-02-11 00:44:22Z jari $

#ifndef _theplu_utility_stl_utility_
#define _theplu_utility_stl_utility_ 

///
/// \file stl_utility.h
///
/// There are a number of useful functionality missing in the Standard
/// Template Library, STL. This file is an effort to provide
/// extensions to STL functionality.
///

#include <string>
#include <utility>
#include <vector>

namespace theplu {
namespace utility {

	///
	/// STL provides operator< for the pair.first element, but none for
	/// pair.second. This template provides this and can be used as the
	/// comparison object in generic functions such as the STL sort.
	///
	template <class T1,class T2>
	struct pair_value_compare
	{
		///
		/// @return true if x.second<y.second or (x.second==y.second and
		/// x.first<y.first)
		///
		inline bool operator()(const std::pair<T1,T2>& x,
													 const std::pair<T1,T2>& y) {
			return ((x.second<y.second) ||
							(!(y.second<x.second) && (x.first<y.first))); 
		}
	};

	///
	/// Function reading from istream to vector of doubles. Function
	/// reads the line until next '\\n'. The line is splitted with
	/// respect to whitespaces and push_backed into the vector. The
	/// vector is emptied before the reading starts. Unexpected
	/// characters are currently skipped with a warning message.
	///
	/// @return false if end of stream
	///
	/// @note The functionality of this function will change in the
	/// future. The overall functionality will be the same but the
	/// outcome of unexpected events will change.
	///
	/// @todo Change the implementation of this function to follow C/C++
	/// style treatment of files and streams.
	///
	bool read_to_double(std::istream&, std::vector<double>&);

	///
	/// Function reading from istream to vector of ints. Function
	/// reads the line until next '\\n'. The line is splitted with
	/// respect to whitespaces and push_backed into the vector. The
	/// vector is emptied before the reading starts. Unexpected
	/// characters are currently skipped with a warning message.
	///
	/// @return false if end of stream
	///
	/// @note The functionality of this function will change in the
	/// future. The overall functionality will be the same but the
	/// outcome of unexpected events will change.
	///
	/// @todo Change the implementation of this function to follow C/C++
	/// style treatment of files and streams.
	///
	bool read_to_int(std::istream&, std::vector<int>&);

	///
	/// Function reading from istream to vector of strings. Function
	/// reads the line until next '\\n'. The line is splitted with
	/// respect to whitespaces and push_backed into the vector. The
	/// vector is emptied before the reading starts.
	///
	/// @return false if end of stream
	///
	bool read_to_string(std::istream&, std::vector<std::string>&);


}} // of namespace utility and namespace theplu

#endif
